/*  Project fancy_ed
    Terry Richards Software
    Copyright  1994. All Rights Reserved.

    SUBSYSTEM:    fancy_ed.exe Application
    FILE:         fnc_mdic.cpp
    AUTHOR:       Terry Richards


    OVERVIEW
    ========
    Source file for implementation of fancy_edMDIClient (TMDIClient).      
*/


#include <owl\owlpch.h>
#pragma hdrstop

#include <dir.h>

#include "fncy_dap.h"
#include "fncy_dwn.h"
#include "fnc_mdi1.h"
#include "fnc_mdic.h"
#include "apxprint.h"
#include "apxprev.h"


//{{fancy_edMDIClient Implementation}}


//
// Build a response table for all messages/commands handled
// by fancy_edMDIClient derived from TMDIClient.
//
DEFINE_RESPONSE_TABLE1(fancy_edMDIClient, TMDIClient)
//{{fancy_edMDIClientRSP_TBL_BEGIN}}
    EV_COMMAND(CM_MDIFILENEW, CmFileNew),
    EV_COMMAND(CM_MDIFILEOPEN, CmFileOpen),
    EV_COMMAND(CM_FILEPRINT, CmFilePrint),
    EV_COMMAND(CM_FILEPRINTERSETUP, CmFilePrintSetup),
    EV_COMMAND(CM_FILEPRINTPREVIEW, CmFilePrintPreview),
    EV_COMMAND_ENABLE(CM_FILEPRINT, CmPrintEnable),
    EV_COMMAND_ENABLE(CM_FILEPRINTERSETUP, CmPrintEnable),
    EV_COMMAND_ENABLE(CM_FILEPRINTPREVIEW, CmPrintEnable),
    EV_WM_DROPFILES,
//{{fancy_edMDIClientRSP_TBL_END}}
END_RESPONSE_TABLE;


//////////////////////////////////////////////////////////
// fancy_edMDIClient
// ===========
// Construction/Destruction handling.
fancy_edMDIClient::fancy_edMDIClient (TModule* module)
 : TMDIClient (module)
{
    // Change the window's background color
    SetBkgndColor(RGB(0xc0, 0xc0, 0xc0));

    ChildCount = 0;

    // INSERT>> Your constructor code here.

}


fancy_edMDIClient::~fancy_edMDIClient ()
{
    Destroy();

    // INSERT>> Your destructor code here.

}


//////////////////////////////////////////////////////////
// fancy_edMDIClient
// ===========
// MDIClient site initialization.
void fancy_edMDIClient::SetupWindow ()
{
    // Default SetUpWindow processing.
    TMDIClient::SetupWindow ();

    fancy_edApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), fancy_edApp);

    // Common file file flags and filters for Open/Save As dialogs.  Filename and directory are
    // computed in the member functions CmFileOpen, and CmFileSaveAs.
    theApp->FileData.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
    theApp->FileData.SetFilter("All Files (*.*)|*.*|");

    // Accept files via drag/drop in the client window.
    DragAcceptFiles(true);
}


//////////////////////////////////////////////////////////
// fancy_edMDIClient
// ===========
// Menu File New command
void fancy_edMDIClient::CmFileNew ()
{
    char    title[255];
    char    numberStr[16];

    // Generate a title for the MDI child window.
    if (GetModule()->LoadString(IDS_UNTITLED, title, sizeof(title))) {
        wsprintf(numberStr, " - %d", ChildCount++);
        strcat(title, numberStr);
    } else
        title[0] = '\0';

    fancy_edMDIChild* child = new fancy_edMDIChild(*this, title, 0);

    // Associate ICON w/ this child window.
    child->SetIcon(GetApplication(), IDI_DOC);

    // If the current active MDI child is maximize then this one should be also.
    fancy_edMDIChild *curChild = (fancy_edMDIChild *)GetActiveMDIChild();
    if (curChild && (curChild->GetWindowLong(GWL_STYLE) & WS_MAXIMIZE))
        child->Attr.Style |= WS_MAXIMIZE;

    child->Create();
}


void fancy_edMDIClient::OpenFile (const char *fileName)
{
    fancy_edApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), fancy_edApp);

    if (fileName)
        strcpy(theApp->FileData.FileName, fileName);

    //
    // Create a MDIChild window whose client is fancy_edWindow.
    //
    //TR - passed title to client window too
    fancy_edMDIChild* child = new fancy_edMDIChild(*this, theApp->FileData.FileName, new fancy_edWindow(0,theApp->FileData.FileName));

    // Associate ICON w/ this child window.
    child->SetIcon(GetApplication(), IDI_DOC);

    // If the current active MDI child is maximize then this one should be also.
    fancy_edMDIChild *curChild = (fancy_edMDIChild *)GetActiveMDIChild();
    if (curChild && (curChild->GetWindowLong(GWL_STYLE) & WS_MAXIMIZE))
        child->Attr.Style |= WS_MAXIMIZE;

    child->Create();

    LoadTextFile();
}


//////////////////////////////////////////////////////////
// fancy_edMDIClient
// ===========
// Menu File Open command
void fancy_edMDIClient::CmFileOpen ()
{
    fancy_edApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), fancy_edApp);

    //
    // Display standard Open dialog box to select a file name.
    //
    *(theApp->FileData.FileName) = 0;
    if (TFileOpenDialog(this, theApp->FileData).Execute() == IDOK)
        OpenFile();
}


// Used by ListBox client to read a text file into the list box.
void fancy_edMDIClient::LoadTextFile ()
{
    char            buf[255+1];
    ifstream        *inStream;

    fancy_edApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), fancy_edApp);

    fancy_edMDIChild  *curChild = (fancy_edMDIChild *)GetActiveMDIChild();
    TListBox        *client = TYPESAFE_DOWNCAST(curChild->GetClientWindow(), TListBox);

    // Only work if the client class is a TListBox.
    if (client) {
        client->ClearList();
        inStream = new ifstream(theApp->FileData.FileName);
        while (inStream->good()) {
            inStream->getline(buf, sizeof(buf) - 1);
            if (inStream->good())
                client->AddString(buf);
        }

        // Return an error message if we had a stream error and it wasn't the eof.
        if (inStream->bad() && !inStream->eof()) {
            string msgTemplate(*GetModule(), IDS_UNABLEREAD);
            char*  msg = new char[MAXPATH + msgTemplate.length()];
            wsprintf(msg, msgTemplate.c_str(), *(theApp->FileData.FileName));
            MessageBox(msg, GetApplication()->GetName(), MB_ICONEXCLAMATION | MB_OK);
            delete msg;
        }

        delete inStream;
    }
}


//////////////////////////////////////////////////////////
// fancy_edMDIClient
// ==========
// Menu File Print command
void fancy_edMDIClient::CmFilePrint ()
{
    //
    // Create Printer object if not already created.
    // 
    fancy_edApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), fancy_edApp);
    if (theApp) {
        if (!theApp->Printer)
            theApp->Printer = new TPrinter(GetApplication());

        //
        // Create Printout window and set characteristics.
        //
        APXPrintOut printout(theApp->Printer, Title, GetActiveMDIChild()->GetClientWindow(), true);

        theApp->Printing++;

        //
        // Bring up the Print dialog and print the document.
        //
        theApp->Printer->Print(GetWindowPtr(GetActiveWindow()), printout, true);

        theApp->Printing--;
    }
}


//////////////////////////////////////////////////////////
// fancy_edMDIClient
// ==========
// Menu File Print Setup command
void fancy_edMDIClient::CmFilePrintSetup ()
{
    fancy_edApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), fancy_edApp);
    if (theApp) {
        if (!theApp->Printer)
            theApp->Printer = new TPrinter(GetApplication());

        //
        // Bring up the Print Setup dialog.
        //
        theApp->Printer->Setup(this);
    }
}


//////////////////////////////////////////////////////////
// fancy_edMDIClient
// ==========
// Menu File Print Preview command
void fancy_edMDIClient::CmFilePrintPreview ()
{
    fancy_edApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), fancy_edApp);
    if (theApp) {
        if (!theApp->Printer)
            theApp->Printer = new TPrinter(GetApplication());

        theApp->Printing++;

        PreviewWindow *prevW = new PreviewWindow(Parent, theApp->Printer, GetActiveMDIChild()->GetClientWindow(), "Print Preview", new TLayoutWindow(0));
        prevW->Create();

        GetApplication()->BeginModal(GetApplication()->GetMainWindow());

        // We must destroy the preview window explicitly.  Otherwise, the window will not be destroyed until
        // it's parent the MainWindow is destroyed.
        prevW->Destroy();
        delete prevW;

        theApp->Printing--;
    }
}


//////////////////////////////////////////////////////////
// fancy_edMDIClient
// ==========
// Menu enabler used by Print, Print Setup and Print Preview.
void fancy_edMDIClient::CmPrintEnable (TCommandEnabler &tce)
{
    if (GetActiveMDIChild()) {
        fancy_edApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), fancy_edApp);
        if (theApp) {
            // If we have a Printer already created just test if all is okay.
            // Otherwise create a Printer object and make sure the printer
            // really exists and then delete the Printer object.
            if (!theApp->Printer) {
                theApp->Printer = new TPrinter(GetApplication());
                
                tce.Enable(theApp->Printer->GetSetup().Error == 0);
            } else
                tce.Enable(theApp->Printer->GetSetup().Error == 0);
        }
    } else
        tce.Enable(false);
}


void fancy_edMDIClient::EvDropFiles (TDropInfo)
{
    Parent->ForwardMessage();
}
